// cSpell:ignore curr
// cSpell:words zlib iconv
import * as fs from 'fs';
import * as Stream from 'stream';
import { fileURLToPath } from 'url';
import { promisify } from 'util';
import * as zlib from 'zlib';
import { decode } from '../../common/encode-decode.mjs';
import { createDecoderTransformer } from '../../common/transformers.mjs';
import { fetch } from './fetch.mjs';
import { FetchUrlError } from './FetchError.mjs';
import { isFileURL, isSupportedURL, isZipped, toURL } from './util.mjs';
const defaultEncoding = 'utf8';
const pipeline = promisify(Stream.pipeline);
export async function readFile(filename, encoding) {
    const url = toURL(filename);
    if (!isSupportedURL(url)) {
        throw new Error('Unsupported network protocol');
    }
    return isFileURL(url) ? _readFile(url, encoding) : _fetchURL(url, encoding);
}
function _readFile(url, encoding) {
    // Convert it to a string because Yarn2 cannot handle URLs.
    const filename = fileURLToPath(url);
    return _read(() => fs.createReadStream(filename), isZipped(filename), encoding);
}
async function _fetchURL(url, encoding) {
    const response = await fetch(url);
    if (!response.ok) {
        throw FetchUrlError.create(url, response.status);
    }
    return _read(() => response.body, isZipped(url), encoding);
}
async function _read(getStream, isZipped, encoding) {
    const stream = getStream();
    const decoder = createDecoderTransformer(encoding);
    const collector = createCollector(encoding || defaultEncoding);
    return isZipped ? pipeline(stream, zlib.createGunzip(), decoder, collector) : pipeline(stream, decoder, collector);
}
export function readFileSync(filename, encoding) {
    const rawData = fs.readFileSync(filename);
    const data = isZipped(filename) ? zlib.gunzipSync(rawData) : rawData;
    return !encoding || encoding.startsWith('utf') ? decode(data) : data.toString(encoding);
}
function createCollector(encoding) {
    async function collect(iterable) {
        const buf = [];
        for await (const sb of iterable) {
            buf.push(typeof sb === 'string' ? sb : sb.toString(encoding));
        }
        return buf.join('');
    }
    return collect;
}
